@@ -4,7 +4,11 @@ use std::sync::{
 };
 use std::time::Duration;
 
-use super::utils::{check_args, fetch_symbols_retry};
+use super::utils::{
+    check_args, fetch_symbols_retry, get_candlestick_intervals, get_connection_interval_ms,
+    get_send_interval_ms,
+};
+use crate::utils::WS_LOCKS;
 use crate::{msg::Message, MessageType};
 use crypto_markets::MarketType;
 use crypto_ws_client::*;
@@ -27,3 +31,5 @@ gen_crawl_event!(crawl_bbo, BitfinexWSClient, MessageType::BBO, subscribe_bbo);
 gen_crawl_event!(crawl_l3_event, BitfinexWSClient, MessageType::L3Event, subscribe_l3_orderbook);
 #[rustfmt::skip]
 gen_crawl_event!(crawl_ticker, BitfinexWSClient, MessageType::Ticker, subscribe_ticker);
+#[rustfmt::skip]
+gen_crawl_candlestick!(crawl_candlestick, BitfinexWSClient);
