@@ -4,7 +4,11 @@ use std::sync::{
 };
 use std::time::Duration;
 
-use super::utils::{check_args, fetch_symbols_retry};
+use super::utils::{
+    check_args, fetch_symbols_retry, get_candlestick_intervals, get_connection_interval_ms,
+    get_send_interval_ms,
+};
+use crate::utils::WS_LOCKS;
 use crate::{msg::Message, MessageType};
 use crypto_markets::MarketType;
 use crypto_ws_client::*;
@@ -23,6 +27,8 @@ gen_crawl_event!(crawl_l2_event_internal, BitmexWSClient, MessageType::L2Event,
 gen_crawl_event!(crawl_bbo_internal, BitmexWSClient, MessageType::BBO, subscribe_bbo);
 #[rustfmt::skip]
 gen_crawl_event!(crawl_l2_topk_internal, BitmexWSClient, MessageType::L2TopK, subscribe_orderbook_topk);
+#[rustfmt::skip]
+gen_crawl_candlestick!(crawl_candlestick_internal, BitmexWSClient);
 
 fn crawl_all(
     msg_type: MessageType,
@@ -168,3 +174,31 @@ pub(crate) fn crawl_funding_rate(
         }
     }
 }
+
+pub(crate) fn crawl_candlestick(
+    market_type: MarketType,
+    symbol_interval_list: Option<&[(String, usize)]>,
+    on_msg: Arc<Mutex<dyn FnMut(Message) + 'static + Send>>,
+    duration: Option<u64>,
+) -> Option<std::thread::JoinHandle<()>> {
+    if market_type == MarketType::Unknown {
+        let on_msg_ext = Arc::new(Mutex::new(move |msg: String| {
+            let message = Message::new(
+                EXCHANGE_NAME.to_string(),
+                MarketType::Unknown,
+                MessageType::Candlestick,
+                msg,
+            );
+            (on_msg.lock().unwrap())(message);
+        }));
+
+        let channels = vec!["tradeBin1m".to_string(), "tradeBin5m".to_string()];
+
+        let ws_client = BitmexWSClient::new(on_msg_ext, None);
+        ws_client.subscribe(channels.as_slice());
+        ws_client.run(duration);
+        None
+    } else {
+        crawl_candlestick_internal(market_type, symbol_interval_list, on_msg, duration)
+    }
+}
