@@ -5,7 +5,11 @@ use std::sync::{
 
 use std::time::Duration;
 
-use super::utils::{check_args, fetch_symbols_retry};
+use super::utils::{
+    check_args, fetch_symbols_retry, get_candlestick_intervals, get_connection_interval_ms,
+    get_send_interval_ms,
+};
+use crate::utils::WS_LOCKS;
 use crate::{msg::Message, MessageType};
 use crypto_markets::MarketType;
 use crypto_ws_client::*;
@@ -63,3 +67,5 @@ gen_crawl_event!(crawl_bbo, DeribitWSClient, MessageType::BBO, subscribe_bbo);
 gen_crawl_event!(crawl_l2_topk, DeribitWSClient, MessageType::L2TopK, subscribe_orderbook_topk);
 #[rustfmt::skip]
 gen_crawl_event!(crawl_ticker, DeribitWSClient, MessageType::Ticker, subscribe_ticker);
+#[rustfmt::skip]
+gen_crawl_candlestick!(crawl_candlestick, DeribitWSClient);
