@@ -4,7 +4,11 @@ use std::sync::{
 };
 use std::time::Duration;
 
-use super::utils::{check_args, fetch_symbols_retry};
+use super::utils::{
+    check_args, fetch_symbols_retry, get_candlestick_intervals, get_connection_interval_ms,
+    get_send_interval_ms,
+};
+use crate::utils::WS_LOCKS;
 use crate::{msg::Message, MessageType};
 use crypto_markets::MarketType;
 use crypto_ws_client::*;
@@ -22,3 +26,5 @@ gen_crawl_event!(crawl_l2_event, KrakenWSClient, MessageType::L2Event, subscribe
 gen_crawl_event!(crawl_bbo, KrakenWSClient, MessageType::BBO, subscribe_bbo);
 #[rustfmt::skip]
 gen_crawl_event!(crawl_ticker, KrakenWSClient, MessageType::Ticker, subscribe_ticker);
+#[rustfmt::skip]
+gen_crawl_candlestick!(crawl_candlestick, KrakenWSClient);
