@@ -5,7 +5,11 @@ use std::sync::{
 
 use std::time::Duration;
 
-use super::utils::{check_args, fetch_symbols_retry};
+use super::utils::{
+    check_args, fetch_symbols_retry, get_candlestick_intervals, get_connection_interval_ms,
+    get_send_interval_ms,
+};
+use crate::utils::WS_LOCKS;
 use crate::{msg::Message, MessageType};
 use crypto_markets::MarketType;
 use crypto_ws_client::*;
@@ -29,6 +33,11 @@ gen_crawl_event!(crawl_ticker_spot, ZbgSpotWSClient, MessageType::Ticker, subscr
 #[rustfmt::skip]
 gen_crawl_event!(crawl_ticker_swap, ZbgSwapWSClient, MessageType::Ticker, subscribe_ticker);
 
+#[rustfmt::skip]
+gen_crawl_candlestick!(crawl_candlestick_spot, ZbgSpotWSClient);
+#[rustfmt::skip]
+gen_crawl_candlestick!(crawl_candlestick_swap, ZbgSwapWSClient);
+
 pub(crate) fn crawl_trade(
     market_type: MarketType,
     symbols: Option<&[String]>,
@@ -73,3 +82,20 @@ pub(crate) fn crawl_ticker(
         _ => panic!("ZBG does NOT have the {} market type", market_type),
     }
 }
+
+pub(crate) fn crawl_candlestick(
+    market_type: MarketType,
+    symbol_interval_list: Option<&[(String, usize)]>,
+    on_msg: Arc<Mutex<dyn FnMut(Message) + 'static + Send>>,
+    duration: Option<u64>,
+) -> Option<std::thread::JoinHandle<()>> {
+    match market_type {
+        MarketType::Spot => {
+            crawl_candlestick_spot(market_type, symbol_interval_list, on_msg, duration)
+        }
+        MarketType::InverseSwap | MarketType::LinearSwap => {
+            crawl_candlestick_swap(market_type, symbol_interval_list, on_msg, duration)
+        }
+        _ => panic!("ZBG does NOT have the {} market type", market_type),
+    }
+}
