@@ -137,3 +137,11 @@ fn test_crawl_ticker(market_type: MarketType, symbol: &str) {
         MessageType::Ticker
     )
 }
+
+#[test_case(MarketType::Spot)]
+#[test_case(MarketType::InverseSwap)]
+#[test_case(MarketType::LinearSwap)]
+#[test_case(MarketType::InverseFuture)]
+fn test_crawl_candlestick(market_type: MarketType) {
+    gen_test_crawl_candlestick!(EXCHANGE_NAME, market_type)
+}
